package Lire::Config::Parser;

use strict;

use vars qw/$VERSION/;

use Lire::Config::Spec;
use Lire::Config::Values;
use XML::Parser;

use Carp;

use vars qw/$LRCML_NS %LRCML_ELEMENTS/;

BEGIN {
    $VERSION = '$Revision: 1.34 $' =~ m!Revision: ([.\d]+)!;
    $LRCML_NS = 'http://www.logreport.org/LRCML/';

    my @elmnts = qw(config global report template job param);

    %LRCML_ELEMENTS = map { $_ => 1 } @elmnts;
}

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my %args = @_;

    croak "missing 'spec' parameter"
      unless $args{'spec'};

    croak "'spec' parameter isn't a Lire::Config::Spec::Config object: $args{'spec'}"
      unless UNIVERSAL::isa( $args{'spec'}, "Lire::Config::Spec::Config" );

    my $self = bless {
        'specstack' => [],
        'confstack' => [],
        'spec' => $args{'spec'},
    }, $class;

    return $self;
}

sub load_config_file {
    my ($self, $file) = @_;

    my $parser = new XML::Parser(
        'Handlers' => {
            'Init' => \&Init,
            'Final' => \&Final,
            'Start' => \&Start,
            'End' => \&End,
            'Char' => \&Char
        },
        'Namespaces' => 1,
        'NoLWP'   => 1,
    );

    # XML::Parser does not provide a real way to pass our own
    # data along, so we'll make do with a hack. sigh.
    $parser->{'LireConfigParser'} = $self;

    $self->{'conf'} = new Lire::Config::ConfigFile( 'spec' => $self->{'spec'},
                                                  'filename' => $file,
                                                );

    open(CONF, $file)
      or croak("can't open Lire Config file $file: $!");

    eval { $parser->parse(\*CONF); };

    croak "error while parsing XML Config $file: $@"
      if $@;

    close CONF;

    return $self->{'conf'};
}

sub Init {
    my ($self) = @_;
}

sub Final {
    my ($self) = @_;
    return $self->{'conf'};
}

sub error {
    my ($self, $expat, $msg) = @_;

    # Remove other at line message
    # $msg =~ s/(at.*?line \d+\n*)//gm;

    my $line = $expat->current_line;

    croak("$msg at line $line");
}

sub Start {
    my ($expat, $name) = @_;

    my $self = $expat->{'LireConfigParser'};

    my $ns = $expat->namespace($name);
    $ns ||= ''; # Remove warning
    if($ns eq $LRCML_NS) {
        # This is one of our elements
        croak($self, "unknown Lire ConfigSpec element: <$name>")
            unless exists $LRCML_ELEMENTS{$name};

        {
            no strict 'refs';

            my $sub = $name . "_start";
            $sub =~ s/-/_/g;    # Hyphen aren't allowed in perl sub names

            eval { $sub->($self, @_); };

            $self->error($expat, $@) if $@;
        };
    }
}

sub End {
    my ($expat, $name) = @_;

    my $self = $expat->{'LireConfigParser'};

    return unless exists $expat->{'LireConfigParser'}; # wtf?

    my $ns = $expat->namespace($name);
    $ns ||= ''; # Remove warning
    if($ns eq $LRCML_NS) {
        # This is one of our element
        croak("unknown Lire Config element: <$name>")
            unless exists $LRCML_ELEMENTS{$name};

        {
            no strict 'refs';

            my $sub = $name . "_end";
            $sub =~ s/-/_/g;    # Hyphen aren't allowed in perl sub names

            eval { $sub->($self, @_); };

            $self->error($expat, $@) if $@;
        }
    }
}

sub Char {
    my ($expat, $text) = @_;

    my $self = $expat->{'LireConfigParser'};

    my $cont = $self->{'confstack'}[0];

    if( defined($cont) && $cont->isa( "Lire::Config::Scalar" ) ) {
        local $SIG{'__WARN__'} = sub {};
        my $old_text = $cont->get();
        if ( defined $text ) {
            $cont->set( $old_text . $text )
        } else {
            $cont->set($text)
        }
    }
}

sub param_start {
    my ($self, $expat, $param, %attr) = @_;

    my $name = $attr{'name'};
    my $pspec = $self->{'specstack'}[0];
    my $pconf = $self->{'confstack'}[0];

    croak "root element must be <config>"
        unless defined($pspec) && defined($pconf);

    my $spec = $pspec->get($name);
    my $conf = $spec->instance(%attr);

    if($pconf->isa("Lire::Config::List")) {
        $pconf->append( $conf );
    } elsif($pconf->isa("Lire::Config::Dictionary")) {
        $pconf->set($conf);
    } else {
        croak "unknown parameter container type: $pconf";
    }

    if($conf->isa('Lire::Config::Scalar')) {
        $conf->init('');
    }

    unshift @{$self->{'specstack'}}, $spec;
    unshift @{$self->{'confstack'}}, $conf;
}

sub param_end {
    my ($self) = @_;
    shift @{$self->{'specstack'}};
    shift @{$self->{'confstack'}};
}

sub config_start {
    my ($self, $expat, $name, %attr) = @_;

    croak "<config> must be the root element"
        if @{$self->{'confstack'}};

    $self->{'specstack'} = [$self->{'spec'}];
    $self->{'confstack'} = undef;
}

sub config_end {
    my ($self) = @_;

    shift @{$self->{'specstack'}};
    shift @{$self->{'confstack'}};

    croak "<config> must be the root element"
        if @{$self->{'confstack'}};
}

sub template_start {
    my ($self, $expat, $template, %attr) = @_;
    my $name = $attr{'name'};

    croak("Template `$name' is already defined")
        if $self->{'conf'}->has_template( $name );

    my $spec = $self->{'specstack'}[0];
    my $conf = $spec->instance(%attr );
    $self->{'conf'}->template( $name, $conf );
    $self->{'confstack'} = [ $conf ];
}

sub template_end {
    my ($self) = @_;

    $self->{'confstack'} = undef;
}

sub job_start {
    my ($self, $expat, $job, %attr) = @_;
    my $name = $attr{'name'};

    croak("Job `$name' is already defined")
        if $self->{'conf'}->has_job( $name );

    my $spec = $self->{'specstack'}[0];
    my $conf = $spec->instance(%attr);
    $self->{'conf'}->job( $name, $conf );
    $self->{'confstack'} = [ $conf ];
}

sub job_end {
    my ($self) = @_;

    $self->{'confstack'} = undef;
}

sub report_start {
    my ($self, $expat, $report, %attr) = @_;
    my $name = $attr{'name'};

    croak("Configuration for report `$name' is already defined")
        if $self->{'conf'}->has_report( $name );

    my $spec = $self->{'specstack'}[0];
    my $serv_conf = $spec->instance(%attr);
    $self->report( $name, $serv_conf );
    $self->{'confstack'} = [ $serv_conf ];
}

sub report_end {
    my ($self) = @_;

    $self->{'confstack'} = undef;
}

sub global_start {
    my ($self, $expat, $global, %attr) = @_;
    my $spec = $self->{'specstack'}[0];
    my $global_conf = $spec->instance(%attr);
    $self->{'conf'}->global( $global_conf );
    $self->{'confstack'} = [ $global_conf ];
}

sub global_end {
    my ($self) = @_;

    $self->{'confstack'} = undef;
}

# keep perl happy
1;

__END__

=pod

=head1 NAME

Lire::Config::Parser -

=head1 SYNOPSIS

Parses a single Lire configuration file.

=head1 DESCRIPTION

Rather straightforward XML parser that follows the configuration
specification to parse an XML file.

=head1 VERSION

$Id: Parser.pm,v 1.34 2003/11/04 17:26:59 flacoste Exp $

=head1 COPYRIGHT

Copyright (C) 2002 Stichting LogReport Foundation LogReport@LogReport.org

This file is part of Lire.

Lire is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see COPYING); if not, check with
http://www.gnu.org/copyleft/gpl.html or write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

=head1 AUTHOR

Wessel Dankers <wsl@logreport.org>

=cut
