package Lire::Config::Spec::Type;

use strict;

=pod

=head1 NAME

Lire::Config::Spec - Package which contains the configuration Spec object

=head1 DESCRIPTION

This package contains all the classes which together can be used to
represent a configuration specification.

=head1 Lire::Config::Spec::Type

Abstract superclass for the other Lire::Config::Spec classes.
Implements basic methods but croaks when those are invoked.


=cut

use vars qw($VERSION);

use Lire::DataTypes qw/:xml/;
use Carp;

BEGIN {
    $VERSION = '$Version:$' =~ m!Revision: ([.\d]+)!;
}

=pod

=head2 new( 'name' => $name )

The constructor takes one mandatory argument which is the parameter's
name.

=cut

sub new {
    my $proto = shift;
    my $class = ref $proto || $proto;
    my %args = @_;

    croak "missing 'name' parameter"
        unless defined $args{'name'};

    croak "name should only contain digits, numbers or '.', '-' or '_': $args{'name'}"
      unless check_xml_name( $args{'name'} );

    my $self = bless {
        'name' => $args{'name'},
        'section' => undef,
        'summary' => undef,
        'description' => undef,
        'contexts' => []
    }, $class;

    $self->section($args{'section'});
    $self->contexts($args{'contexts'});

    return $self;
}

=pod

=head2 name()

Returns the parameter's name.

=cut

sub name {
    $_[0]{'name'};
}

=pod

=head2 has_context()

Returns if this parameter is interesting for the given context.

=cut

sub has_context {
    my $self = shift;
    return 1 if !%{$self->{'contexts'}};
    foreach(@_) {
        s/^\s+|\s+$//g;
        return exists $self->{'contexts'}{lc $_};
    }
}

=pod

=head2 contexts()

Returns a list of contexts of this parameter. If an argument is passed, it is
used to change the list of contexts. It's ok to pass undef as a parameter
(treated as '').

=cut

sub contexts {
    my $self = shift;

    return keys %{$self->{'contexts'}}
      unless @_;

    my %contexts;

    @contexts{split(' ', lc)} = ()
      foreach @_;

    $self->{'contexts'} = \%contexts;

    return keys %contexts;
}

=pod

=head2 has_section()

Returns if this parameter is interesting for the given section.

=cut

sub has_section {
    my $self = shift;
    foreach(@_) {
        if(defined) {
            s/^\s+|\s+$//g;
            return lc eq $self->section;
        } else {
            return !defined $self->section;
        }
    }
}

=pod

=head2 section()

Returns a the section of this parameter. If an argument is passed, it is
used to change the section. It's ok to pass undef as a parameter
(treated as "not a member of any section").

=cut

sub section {
    my $self = shift;

    foreach(@_) {
        if(defined) {
            s/^\s+|\s+$//g;
            $self->{'section'} = lc;
        } else {
            $self->{'section'} = undef;
        }
    }

    return $self->{'section'};
}

=pod

=head2 summary( [$summary] )

Returns this parameter summary description. It can also be used
to change the parameter's summary.

=cut

sub summary {
    my $self = $_[0];

    if (@_ == 2) {
        $self->{'summary'} = $_[1];
    }
    $self->{'summary'};
}

=pod

=head2 description( [$description] )

Returns this parameter's description. This is a DocBook string. It can
also be used to change the parameter's description.

=cut

sub description {
    my $self = $_[0];

    if (@_ == 2) {
        croak "'description' parameter should be a DocBook string"
          unless $_[1] =~ /^\s*<[\w.-]+>/;

        $self->{'description'} = $_[1];
    }

    $self->{'description'};
}

=pod

=head2 instance()

This method is a factory type function which returns
a new Lire::Config::Value object of the correct type.

=cut

sub instance {
    croak("instance() method not implemented in ", ref $_[0] );
}


=pod

=head2 is_instance( $value_object )

Returns true if $value_object is a valid instance of this parameter.

=cut

sub is_instance {
    my ( $self, $instance ) = @_;

    croak "missing 'instance' parameter"
      unless defined $instance;

    croak "'instance' parameter should be a Lire::Config::Value: $instance"
      unless UNIVERSAL::isa( $instance, 'Lire::Config::Value' );

    return $instance->spec eq $self;
}

sub instance_class_ {
    croak("instance_class_() method not implemented in ", ref $_[0] );
}

package Lire::Config::Spec::Scalar;

=pod

=head1 Lire::Config::Spec

Describes an abstract type for scalar configuration variables.

=cut

use base qw(Lire::Config::Spec::Type);
use Lire::Config::Values;
use Carp;

sub instance {
    my $self = shift;
    return new Lire::Config::Scalar( @_, 'spec' => $self );
}

sub instance_class_ {
    return "Lire::Config::Scalar";
}

=pod

=head2 is_valid( $value )

Returns true if $value is appropriate for that parameter type. $value
is the value has entered by the user for example.

=cut

sub is_valid {
    croak("is_valid() method not implemented in ", ref $_[0] );
}

=pod

=head2 normalize( $value )

This method returns the $value normalized for this type.

=cut

sub normalize {
    croak("normalize() method not implemented in ", ref $_[0] );
}

package Lire::Config::Spec::Compound;

=pod

=head1 Lire::Config::Spec::Compound

Describes an abstract type for configuration variables that can
contain subtypes.

=cut

use base qw(Lire::Config::Spec::Type);
use Lire::Config::Values;
use Carp;

sub new {
    my $self = shift->SUPER::new(@_);

    $self->{'components'} = {};

    return $self;
}

=pod

=head2 has_component( $name )

Returns true if this type has a component named $name.

=cut

sub has_component {
    my ( $self, $name ) = @_;

    croak "missing 'name' parameter"
      unless defined $name;

    return exists $self->{'components'}{$name};
}

=pod

=head2 components()

Returns the subtypes contained in this compound type.

=cut

sub components {
    return values %{$_[0]{'components'}}
}

=pod

=head2 add( $type )

This method adds $type as a component of this type.

=cut

sub add {
    my ($self, $type) = @_;

    croak "missing 'type' parameter"
      unless defined $type;

    croak "'type' parameter should be a Lire::Config::Spec::Type instance, not $type"
      unless UNIVERSAL::isa( $type, "Lire::Config::Spec::Type" );

    croak( $self->name, " already contains a component named '", $type->name, "'" )
      if exists $self->{'components'}{$type->name};

    $self->{'components'}->{$type->name} = $type;
}

=pod

=head2 get($name)

The get() method returns the component of this type with the
requested name.

=cut

sub get {
    my ($self, $name) = @_;

    croak "missing 'name' parameter"
      unless $name;

    croak "no component named '$name' in type $self->{'name'}"
      unless exists $self->{'components'}{$name};

    return $self->{'components'}{$name};
}

package Lire::Config::Spec::List;

=pod

=head1 Lire::Config::Spec::List

Describes a type of configuration variables that represent a list
of other types.

=cut

use base qw(Lire::Config::Spec::Compound);

use Lire::Config::Values;

sub instance {
    my $self = shift;
    return new Lire::Config::List(@_, 'spec' => $self );
}

sub instance_class_ {
    return "Lire::Config::List";
}

package Lire::Config::Spec::Config;

=pod

=head1 Lire::Config::Spec::Config

Describes a type of configuration variables that contain a configuration.
That is, not the entire configuration but a part, such as a single
template or the list of global configuration items.

=cut

use base qw(Lire::Config::Spec::Compound);
use Carp;

sub new {
    return shift->SUPER::new( 'name' => 'config', @_ );
}

sub instance {
    my $self = shift;
    return new Lire::Config::Dictionary(@_, 'spec' => $self );
}

sub instance_class_ {
    return "Lire::Config::Dictionary";
}

package Lire::Config::Spec::String;

=pod

=head1 Lire::Config::Spec::String

Type that represents any string of characters Values of this type can
be any string.

There's not much to check about these values; if they are already a valid
perl string they are also a valid configuration string.

=cut

use base qw(Lire::Config::Spec::Scalar);

sub is_valid {
    return defined $_[1];
}

sub normalize {
    return $_[1];
}

package Lire::Config::Spec::File;

=pod

=head1 Lire::Config::Spec::File

Type that represents a file name. Values of this type are names that
refer to existing files.

File values are only valid when the path they point to exists.

=cut

use base qw(Lire::Config::Spec::Scalar);

use Lire::Utils qw(tilde_expand);
use File::Spec;

sub is_valid {
    my ( $self, $val ) = @_;

    my $file = $self->normalize( $val );

    return 0 unless defined $file;
    return -f $file;
}

sub normalize {
    my ( $self, $val ) = @_;

    return undef unless defined $val;

    return File::Spec->canonpath( tilde_expand( $val ));
}

package Lire::Config::Spec::Service;

=pod

=head1 Lire::Config::Spec::Service

Type that represents a Lire service name. Values of this type are names that
refer to any of the Lire services.

=cut

use base qw(Lire::Config::Spec::Scalar);

use Lire::DlfConverterManager;

sub is_valid {
    my ( $self, $val ) = @_;

    return 0 unless defined $val;

    return defined
      Lire::DlfConverterManager->instance()->get_converter( $self->normalize($val) );
}

sub normalize {
    my ( $self, $val ) = @_;

    return undef unless defined $val;

    $val =~ s/(^\s+|\s$)//;

    return $val;
}

package Lire::Config::Spec::Command;

=pod

=head1 Lire::Config::Spec::Command

Type that represents a UNIX command or program.
A path lookup is done to canonicalize it, if appropriate.

=cut

use base qw(Lire::Config::Spec::Scalar);

use File::Spec;

sub is_valid {
    my ( $self, $val ) = @_;

    my $file = $self->normalize( $val );

    return 0 unless defined $file;
    return -x $file;
}

sub normalize {
    my ( $self, $val ) = @_;

    return undef unless defined $val;

    return File::Spec->canonpath($val)
        if index($val, '/') >= 0;

    foreach (split(':', $ENV{'PATH'})) {
        $_ = '.' if $_ eq '';
        my $p = File::Spec->canonpath("$_/$val");
        return $p if -x $p && -f _;
    }
    return undef;
}

package Lire::Config::Spec::Executable;

=pod

=head1 Lire::Config::Spec::Executable

Type that represents an executable.

Executable values are only valid when the path they point to exists.

=cut

use base qw(Lire::Config::Spec::Scalar);

use Lire::Utils qw(tilde_expand);
use File::Spec;

sub is_valid {
    my ( $self, $val ) = @_;

    return 0 unless defined $val;

    my $file = $self->normalize( $val );
    return -x $file && -f _;
}

sub normalize {
    my ( $self, $val ) = @_;

    return undef unless defined $val;

    my $exe = tilde_expand( $val );

    unless ( index($val, "/") != -1 ) {
        foreach my $d ( File::Spec->path ) {
            if (-f "$d/$exe" && -x "$d/$exe" ) {
                $exe = "$d/$exe";
                last;
            }
        }
    }

    return File::Spec->canonpath( $exe );
}

package Lire::Config::Spec::Directory;

=pod

=head1 Lire::Config::Spec::Directory

Type that represents a directory.

Directory values are valid only when the path they point to exists.

=cut

use base qw(Lire::Config::Spec::Scalar);
use Lire::Utils qw(tilde_expand);
use File::Spec;

sub is_valid {
    my ( $self, $val ) = @_;

    return 0 unless defined $val;
    my $dir = $self->normalize( $val );

    return -d $dir;
}

sub normalize {
    my ( $self, $val ) = @_;

    return undef unless defined $val;

    return File::Spec->canonpath( tilde_expand( $val ) );
}

package Lire::Config::Spec::Boolean;

=pod

=head1 NAME

Lire::Config::Spec::Boolean - Type that represents a boolean

=head1 DESCRIPTION

Values of this type are yes/no values.

These values are normalized to 0 or 1.

=cut

use base qw(Lire::Config::Spec::Scalar);

sub is_valid {
    my ( $self, $val ) = @_;

    return 0 if !defined $val;

    return 1 if !$val;

    return scalar $val =~ /^(0|1|f|false|true|t|no|off|disabled|enabled|yes|on)$/i;
}

sub normalize {
    my ( $self, $val ) = @_;

    return undef unless defined $val;

    return ! 1 if !$val;

    return ! scalar $val =~ /^(0|f|false|no|off|disabled)$/i;
}

package Lire::Config::Spec::Integer;

=pod

=head1 Lire::Config::Spec::Integer

Type that represents an integer number. Values of this type are simple
integer numbers with a range of -2147483648 to 2147483647.

=cut

use base qw(Lire::Config::Spec::Scalar);

sub is_valid {
    my ( $self, $val ) = @_;

    return 0 unless defined $val;

    return scalar $val =~ /^-?[0-9_]+$/;
}

sub normalize {
    my ( $self, $val ) = @_;

    return undef unless defined $val;

    return int($val);
}

package Lire::Config::Spec::Select;

=pod

=head1 Lire::Config::Spec::Select

Type that represents a choice. Values of this type can have one from a
set of possible choices.

The these values are normalized to the case specified in the options'
specification.

=cut

use base qw(Lire::Config::Spec::Compound);

use Lire::Config::Values;
use Carp;

=pod

=head2 options()

Returns as an array the valid options for this select.

=cut

sub options {
    return values %{$_[0]{'components'}};
}

sub instance {
    my $self = shift;
    return new Lire::Config::Scalar( @_, 'spec' => $self );
}

sub add {
    my ($self, $type) = @_;

    croak "missing 'option' parameter"
      unless defined $type;

    croak "'option' parameter should be a Lire::Config::Spec::Option instance, not $type"
      unless UNIVERSAL::isa( $type, "Lire::Config::Spec::Option" );

    my $existing = $self->normalize( $type->name );
    croak( "'", $self->name, "' already contains an option named '", $existing, "'" )
      if defined $existing;

    $self->{'components'}->{$type->name} = $type;
}

sub is_valid {
    my ( $self, $value ) = @_;

    return defined $self->normalize( $value );
}

sub normalize {
    my ($self, $value) = @_;

    return undef unless defined $value;

    foreach my $o ( $self->options ) {
        return $o->name if $o->is_option( $value );
    }

    return undef;
}

package Lire::Config::Spec::Option;

=head1 Lire::Config::Spec::Option

Type that represents a specific choice

=cut

use base qw(Lire::Config::Spec::Type);
use Carp;

=pod

=head2 is_option( $value )

Returns true if $value is equals to this option name.

=cut

sub is_option {
    my ( $self, $value ) = @_;

    return lc $self->{'name'} eq lc $value;
}

1; # whine, whine


__END__

=pod

=head1 VERSION

$Id: Spec.pm,v 1.54 2003/11/06 17:18:39 flacoste Exp $

=head1 COPYRIGHT

Copyright (C) 2002 Stichting LogReport Foundation LogReport@LogReport.org

This file is part of Lire.

3Lire is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see COPYING); if not, check with
http://www.gnu.org/copyleft/gpl.html or write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

=head1 AUTHOR

Wessel Dankers <wsl@logreport.org>

=cut


