package Lire::Config::Value;

=pod

=head1 NAME

Lire::Config::Value

=head1 DESCRIPTION

This package contains the classes use to represent an actual
configuration.

=head1 Lire::Config::Value

Value containers for configuration variables. This is the abstract
superclass for the other Lire::Config::Values classes.

Implements a few simple methods which forward the work to the spec to
which they belong.

=cut

use strict;

use vars qw/$VERSION/;

use Carp;

BEGIN {
    $VERSION = '$Revision: 1.46 $' =~ m!Revision: ([.\d]+)!;
}

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %args = @_;

    croak( "missing 'spec' parameter" )
        unless defined $args{'spec'};

    croak "'spec' parameter should be a Lire::Config::Spec::Type instance"
        unless UNIVERSAL::isa( $args{'spec'}, "Lire::Config::Spec::Type" );

    undef $args{'name'}
        if defined $args{'name'} && $args{'name'} eq $args{'spec'}->name;

    return bless {
        'spec' => $args{'spec'},
        'name' => $args{'name'}
    }, $class;
}

=pod

=head2 name()

Returns this configuration parameter's name.

=cut

sub name {
    my $self = $_[0];
    return defined $self->{'name'} ? $self->{'name'} : $self->{'spec'}->name;
}

=pod

=head2 spec()

Returns the specification for this parameter.

=cut

sub spec {
    return $_[0]{'spec'};
}

=pod

=head2 as_value()

Returns this value as a perl native value. This will be either a
scalar value, or an hash or array references. The returned values
should be normalized.

=cut

sub as_value {
    croak ref $_[0], "::as_value is unimplemented";
}

=pod

=head2 as_shell_var()

Returns this configuration variable's value in a form that can be
evaled in a shell script.

=cut

sub as_shell_var {
    return $_[0]->name . q{="Variable's type not supported in shell"};
}

=pod

=head2 save_xml()

Writes the configuration variable recursively to a file descriptor
reference.

=cut

sub save_xml {
    my $self = shift;
    my ($fh, $indent) = @_;
    if($self->has_value) {
        print $fh '  'x$indent, '<param name="', $self->name, '">', "\n";
        $self->save_value(@_);
        print $fh '  'x$indent, "</param>\n";
    }
}

package Lire::Config::Scalar;

=pod

=head1 Lire::Config::Scalar

Value container for scalar values. Implements simple methods to get()
and set() string data.

=cut

use Carp;
use Lire::Utils qw/ xml_encode shell_quote/;

use base qw/Lire::Config::Value/;

sub new {
    my $proto = shift;
    my $class = ref $proto || $proto;
    my $self = $class->SUPER::new(@_);
    my %args = @_;

    $self->{'value'} = undef;
    $self->{'warned'} = 0;

    $self->set( $args{'value'} )
        if exists $args{'value'};

    return $self;
}

sub as_value {
    my $self = $_[0];
    my $val = $self->spec->normalize($self->{'value'});

    return $val if $self->spec->is_valid($val);

    unless($self->{'warned'}) {
        carp "invalid value for parameter '", $self->name, "': ",
          ( defined $val ? $val : '(undef)' );
        $self->{'warned'} = 1;
    }

    return wantarray ? () : undef;
}

sub as_shell_var {
    my $self = $_[0];

    if ( $self->spec->is_valid( $self->{'value'} ) ) {
        return $self->name . "=" . shell_quote( $self->as_value );
    } else {
        return '';
    }
}

=pod

=head2 has_value()

Returns true if this variable's value as set.

=cut

sub has_value {
    return defined $_[0]->{'value'};
}

=pod

=head2 get()

Returns this scalar's value.

=cut

sub get {
    return $_[0]{'value'};
}

=pod

=head2 set( $val )

This method can be used to change this scalar's value. An exception
will be thrown if $val isn't appropriate for this parameter.

=cut

sub set {
    my ($self, $val) = @_;

    carp "invalid value for parameter '", $self->name, "': ", $val
        unless !defined $val || $self->spec->is_valid($val);

    $self->{'warned'} = 0;

    $self->{'value'} = $val;
}

sub init {
    my ($self, $val) = @_;

    if(defined $self->{'value'}) {
        warn "BUG: parameter ", $self->name, " was already assigned a value";
    } else {
        $self->{'value'} = $val;
    }
}

sub save_xml {
    my $self = shift;
    my ($fh, $indent) = @_;
    if(defined $self->{'value'}) {
        print $fh '  'x$indent, '<param name="', $self->name, "\">";
        print $fh xml_encode($self->{'value'}), "</param>\n";
    }
}

package Lire::Config::Dictionary;

=pod

=head1 Lire::Config::Dictionary

Configuration parameter which contain a bunch of named parameters
other values accessible by name.

This value is used for top-level configuration sets.

=cut

use Carp;

use base qw/Lire::Config::Value/;

sub new {
    my $proto = shift;
    my $class = ref $proto || $proto;
    my $self = $class->SUPER::new(@_);

    croak "'spec' parameter should be a Lire::Config::Spec::Compound instance"
        unless $self->{'spec'}->isa( "Lire::Config::Spec::Compound" );

    $self->{'values'} = {};

    return $self;
}

sub as_value {
    my $self = $_[0];

    my $ret = {};
    foreach my $param ( $self->spec->components ) {
        $ret->{$param->name} = $self->get( $param->name )->as_value;
    }

    return $ret;
}

=pod

=head2 is_set( $name )

Returns true if the parameter $name was set in this dictionary.

=cut

sub is_set {
    my ( $self, $name ) = @_;

    croak "missing 'name' parameter"
      unless defined $name;

    croak "no parameter $name defined in ", $self->name
      unless $self->spec->has_component( $name );

    return exists $self->{'values'}{$name};
}

=pod

=head2 get( $name )

Returns the Value object for the $name subparameter. An exception will
be thrown if there is no subparameter named $name.

=cut

sub get {
    my ($self, $name) = @_;

    croak "missing 'name' parameter"
      unless defined $name;

    croak "no parameter $name defined in ", $self->name
      unless $self->spec->has_component( $name );

    unless ( exists $self->{'values'}{$name} ) {
        # Create empty var
        $self->{'values'}{$name} = $self->spec->get( $name )->instance
    }

    return $self->{'values'}{$name};
}

sub get_all {
    return {%{$_[0]->{'values'}}};
}

=pod

=head2 set( $value )

Sets a parameter in this dictionary.

=cut

sub set {
    my ($self, $val) = @_;

    croak "missing 'value' parameter"
      unless defined $val;

    croak "value ($val) should be an instance of Lire::Config::Value"
      unless UNIVERSAL::isa( $val, "Lire::Config::Value" );

    croak $self->name, " cannot contains a ", $val->name, " parameter"
      unless $self->spec->has_component( $val->name );

    croak "$val is not a valid instance for parameter ", $val->name
      unless $self->spec->get( $val->name )->is_instance( $val );

    $self->{'values'}{$val->name} = $val;
}

sub save_value {
    my $self = shift;
    my ($fh, $indent) = @_;

    while(my ($key, $val) = each(%{$self->{'values'}})) {
        $val->save_xml($fh, $indent+1);
    }
}

sub has_value {
    return defined $_[0]->{'values'} && %{$_[0]->{'values'}};
}

=pod

=head1 Lire::Config::List

Used for configuration parameters which contains a list of values.

=cut

package Lire::Config::List;

use base qw/Lire::Config::Value/;

use Carp;

sub new {
    my $proto = shift;
    my $class = ref $proto || $proto;
    my $self = $class->SUPER::new(@_);

    croak "'spec' parameter should be a Lire::Config::Spec::List instance"
      unless $self->{'spec'}->isa( "Lire::Config::Spec::List" );

    $self->{'elements'} = [];

    return $self;
}

sub as_value {
    return [ map { $_->as_value } @{$_[0]->{'elements'}} ];
}

sub has_value {
    return defined $_[0]->{'elements'} && @{$_[0]->{'elements'}};
}

=pod

=head2 elements()

Returns this list's values as an array of Lire::Config::Value object.

=cut

sub elements {
    return @{$_[0]->{'elements'}};
}

=pod

=head2 get( $idx )

Returns the value stored at index $idx of this list. An exception is
thrown if the index is out-of-bounds.

=cut

sub get {
    my ($self, $idx) = @_;

    croak "missing 'idx' parameter"
      unless defined $idx;

    croak "'idx' parameter should be an integer: $idx"
      unless $idx =~ /^-?\d+$/;

    croak "index out of bounds: $idx"
      unless $idx >= 0 ? ( $idx < @{$self->{'elements'}} ):
                         ( abs($idx)-1 < @{$self->{'elements'}} );

    $self->{'elements'}[$idx];
}

=head2 remove( $idx )

Removes the value stored at index $idx of this list. An exception is
thrown if the index is out-of-bounds. It returns the deleted element.

=cut

sub remove {
    my ($self, $idx) = @_;

    croak "missing 'idx' parameter"
      unless defined $idx;

    croak "invalid index representation"
      unless $idx =~ /^-?\d+$/;

    croak "index out of bounds: $idx"
    unless $idx >= 0 ? ( $idx < @{$self->{'elements'}} ):
                         ( abs($idx)-1 < @{$self->{'elements'}} );

    return splice(@{$self->{'elements'}}, $idx, 1);
}

=pod

=head2 append( $val )

Adds $val to the end of this list.

=cut

sub append {
    my ($self, $val) = @_;

    croak "missing 'value' parameter"
      unless defined $val;

    croak "value parameter should be a Lire::Config::Value instance"
      unless UNIVERSAL::isa( $val, "Lire::Config::Value" );

    croak $self->name, " cannot contains ", $val->name, " parameters"
      unless $self->spec->has_component( $val->name );

    croak "$val is not a valid instance for component ", $val->name
      unless $self->spec->get( $val->name )->is_instance( $val );

    push @{$self->{'elements'}}, $val;
}

sub save_value {
    my $self = shift;
    my ($fh, $indent) = @_;

    foreach my $val (@{$self->{'elements'}}) {
        $val->save_xml($fh, $indent+1);
    }
}

package Lire::Config::ConfigFile;

use Carp;

=pod

=head2 new( 'filename' => $file, spec => $config_spec )

Creates a new Lire::Config::ConfigFile object. The constructor
takes two parameters:

=over

=item filename

The file form where the configuration was stored or was loaded from.

=item spec

The Lire::Config::Spec object to which this configuration adheres.

=end

=cut

sub new {
    my ( $proto, %args ) = @_;
    my $class = ref $proto || $proto;

    croak "missing 'spec' parameter"
      unless defined $args{'spec'};

    croak "'spec' parameter isn't an instance of Lire::Config::Spec: $args{'spec'}"
      unless UNIVERSAL::isa( $args{'spec'}, "Lire::Config::Spec::Config" );

    croak "missing 'filename' parameter"
      unless defined $args{'filename'};

    my $self = bless {
        'spec'      => $args{'spec'},
        'filename'  => $args{'filename'},
        'global'    => undef,
        'templates' => {},
        'reports'  => {},
        'jobs'      => {},
    }, $class;

    return $self;
}

=pod

=head2 filename()

Returns the filename where this configuration file should be stored.

=cut

sub filename {
    $_[0]{'filename'};
}

=pod

=head2 spec()

Returns the Lire::Config::Spec object to which this configuration file
adheres.

=cut

sub spec {
    $_[0]{'spec'};
}

=pod

=head2 global( [ $config] )

Returns or modify the 'global' configuration part of the file. This is
an instance Lire::Config::Compound object.

=cut

sub global {
    my ( $self, $config ) = @_;

    if ( @_ == 2 ) {
        if ( defined $config ) {
            croak "'config' parameter isn't an Lire::Config::Dictionary instance: $config"
              unless UNIVERSAL::isa( $config, "Lire::Config::Dictionary" );
        }
        $self->{'global'} = $config;
    }

    return $self->{'global'};
}

sub _hash_config_names {
    my ( $self, $type ) = @_;

    return keys %{$self->{$type}};
}

sub _hash_config {
    my ( $self, $type, $name, $config ) = @_;

    croak "missing '$name' parameter"
      unless defined $name;
    if ( @_ == 4 ) {
        if ( defined $config ) {
            croak "'config' parameter isn't an Lire::Config::Dictionary instance: $config"
              unless UNIVERSAL::isa( $config, "Lire::Config::Dictionary" );

            $self->{$type}{$name} = $config;
        } else {
            delete $self->{$type}{$name};
        }
    }

    return $self->{$type}{$name};
}

=pod

=head2 has_template( $name )

Returns true if there is a configuration specified for the template $name.

=cut

sub has_template {
    my ($self, $name) = @_;

    croak "missing '$name' parameter"
      unless defined $name;

    return defined $self->{'templates'}{$name};
}


=pod

=head2 template_names()

Returns the name of configuration templates defined in the file.

=cut

sub template_names {
    my $self = $_[0];

    return $self->_hash_config_names( "templates" )
}


=pod

=head2 template( [ $name, $config ] )

Returns or modify the configuration that should be used when the $name
template is in effect.

=cut

sub template {
    my $self = shift;

    return $self->_hash_config( "templates", @_ )
}

=pod

=head2 has_report( $name )

Returns true if there is a configuration specified for the report $name.

=cut

sub has_report {
    my ($self, $name) = @_;

    croak "missing '$name' parameter"
      unless defined $name;

    return defined $self->{'reports'}{$name};
}

=pod

=head2 report_names()

Returns the name of report which have special configuration defined
in the file.

=cut

sub report_names {
    my $self = $_[0];

    return $self->_hash_config_names( "reports" )
}


=pod

=head2 report( [ $name, $config ] )

Returns or modify the configuration that should be used when the $name
report is in effect.

=cut

sub report {
    my $self = shift;

    return $self->_hash_config( "reports", @_ )
}

=pod

=head2 has_job( $name )

Returns true if there is a configuration specified for the job $name.

=cut

sub has_job {
    my ($self, $name) = @_;

    croak "missing '$name' parameter"
      unless defined $name;

    return defined $self->{'jobs'}{$name};
}

=pod

=head2 job_names()

Returns the job's names that have configuration data defined in the
file.

=cut

sub job_names {
    my $self = $_[0];

    return $self->_hash_config_names( "jobs" )
}


=pod

=head2 job( [ $name, $config ] )

Returns or modify the configuration that should be used when the $name
job is in effect.

=cut

sub job {
    my $self = shift;

    return $self->_hash_config( "jobs", @_ )
}

sub save_global {
    my $self = shift;
    my ($fh, $indent) = @_;

    print $fh '  'x$indent, "<global>\n";
    $self->global->save_value($fh, $indent);
    print $fh '  'x$indent, "</global>\n";
}

sub save_reports {
    my $self = shift;
    my ($fh, $indent) = @_;

    foreach ($self->report_names) {
        print $fh '  'x$indent, "<report name=\"$_\">\n";
        $self->report($_)->save_value($fh, $indent);
        print $fh '  'x$indent, "</report>\n";
    }
}

sub save_templates {
    my $self = shift;
    my ($fh, $indent) = @_;

    foreach ($self->template_names) {
        print $fh '  'x$indent, "<template name=\"$_\">\n";
        $self->template($_)->save_value($fh, $indent);
        print $fh '  'x$indent, "</template>\n";
    }
}

sub save_jobs {
    my $self = shift;
    my ($fh, $indent) = @_;

    foreach ($self->job_names) {
        print $fh '  'x$indent, "<job name=\"$_\">\n";
        $self->job($_)->save_value($fh, $indent);
        print $fh '  'x$indent, "</job>\n";
    }
}

sub save_xml {
    my $self = shift;
    my $fh = $_[0];
    print $fh join("\n",
        '<?xml version="1.0" encoding="UTF-8"?>', # hard to say what charset this should be
        '<!DOCTYPE config PUBLIC',
        '  "-//LogReport.ORG//DTD Lire Report Configuration Markup Language V1.0//EN"',
        '  "http://www.logreport.org/LRCML/1.0/lrcml.dtd"[',
        '<!ENTITY % LIRE.pfx "">',
        ']>',
        '<config xmlns="http://www.logreport.org/LRCML/">', '');

    $self->save_global($fh, 1);
    $self->save_reports($fh, 1);
    $self->save_templates($fh, 1);
    $self->save_jobs($fh, 1);

    print $fh "</config>\n";
}

1; # whine, whine

__END__

=pod

=head1 VERSION

$Id: Values.pm,v 1.46 2003/10/29 06:19:18 wsourdeau Exp $

=head1 COPYRIGHT

Copyright (C) 2002-2003 Stichting LogReport Foundation LogReport@LogReport.org

This file is part of Lire.

Lire is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see COPYING); if not, check with
http://www.gnu.org/copyleft/gpl.html or write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

=head1 AUTHOR

Wessel Dankers <wsl@logreport.org>

=cut
