package Lire::DlfConverterProcess;

use strict;

use vars qw/ $VERSION /;

use Lire::DlfConverterManager;
use Symbol;

use POSIX qw/strftime/;

use Carp;

BEGIN {
    ($VERSION)	= '$Revision: 1.3 $' =~ m!Revision: ([.\d]+)!;
};

=pod

=head1 NAME

Lire::DlfConverterProcess - Object that controls the DLF conversion process

=head1 SYNOPSIS

  use Lire::LogSource;
  use Lire::DlfStore;
  use Lire::DlfConverterProcess;

  my $src = new Lire::LogSource( "file", 'pattern' => "/var/log/messages" );
  my $store = Lire::DlfStore->open( "store" );

  my $process = new Lire::DlfConverterProcess( $src, $store );

  $process->convert_source;

  print "Log lines read: ",      $process->line_count(), "\n";
  print "DLF records created: ", $process->dlf_count,    "\n";
  print "Errors encountered: ",  $process->errors_count, "\n";
  print "Ignored records: ",     $process->ignored_count,"\n";

=head1 DESCRIPTION

This object encapsulates the Lire DLF conversion process. It takes as
parameter a Lire::LogSource and a Lire::DlfStore. It will setup the
converter and will converter the content of Lire::LogSource to DLF
which will be saved in the Lire::DlfStore.

The object provides the API to the converter. Methods are also
available to query information on the conversion process.

=head1 new( $src, $store );

Create a Lire::DlfConverterProcess that can be used to convert the
data from Lire::LogSource into DLF records which will be stored into
Lire::DlfStore.

=cut

sub new {
    my ( $pkg, $src, $store ) = @_;

    croak "missing src parameter" unless defined $src;
    croak "missing store parameter" unless defined $store;
    croak "src parameter isn't an instance of Lire::LogSource: $src"
      unless $src->isa( "Lire::LogSource" );
    croak "store parameter isn't an instance of Lire::DlfStore: $store"
      unless $store->isa( "Lire::DlfStore" );

    bless { '_src'   => $src,
            '_store' => $store,
            '_convert_called' => 0,
          }, ref $pkg || $pkg;
}

=pod

=head2 convert_source()

Convert the LogSource to DLF. This method will throw an exception if
it is called more than once.

=cut

sub convert_source {
    my $self = $_[0];

    croak $self->{'_src'}->name, "log source was already processed"
      if $self->{'_convert_callede'};

    $self->_init_conversion_process;

    $self->{'_converter'}->init_dlf_converter( $self );

    if ( $self->{'_converter'}->handle_log_lines ) {
        $self->_handle_continuation();
        $self->_process_log_lines( $self->{'_fh'} );
    } else {
        $self->{'_converter'}->process_log_file( $self, $self->{'_fh'} );
    }
    close $self->{'_fh'};

    $self->{'_converter'}->finish_conversion( $self );

    # Close the dlf streams and other storage
    foreach my $s ( values %{$self->{'_streams'}} ) {
        $s->close;
    }
    close $self->{'_error_fh'};
    close $self->{'_ignored_fh'}
      if $self->{'_ignored_fh'};
    close $self->{'_continuation_fh'}
      if $self->{'_continuation_fh'};

    $self->{'_convert_called'} = 1;
}

#------------------------------------------------------------------------
# Method _init_conversion_process()
#
# This method takes care of initializing the state of the conversion
# process
sub _init_conversion_process {
    my $self = $_[0];

    # Get converter
    my $cname = $self->{'_src'}->converter;
    croak "log source ", $self->{'_src'}->name, " doesn't have a converter defined"
      unless defined $cname;

    my $mgr = Lire::DlfConverterManager->instance;
    $self->{'_converter'} = $mgr->get_converter( $cname );
    croak "DLF converter $cname isn't available"
      unless $self->{'_converter'};

    # Open the appropriate dlf streams
    foreach my $s ( $self->{'_converter'}->schemas ) {
        $self->{'_streams'}{$s} = $self->{'_store'}->open_dlf_stream( $s, "w" );
    }

    # Initialize the counters
    $self->{'_line_count'}    = 0;
    $self->{'_dlf_count'}     = 0;
    $self->{'_ignored_count'} = 0;
    $self->{'_error_count'}   = 0;
    $self->{'_saved_count'}   = 0;

    # Open the log file
    my $file = $self->{'_src'}->file;
    croak "log source ", $self->{'_src'}->name, " doesn't have a file defined"
      unless defined $file;
    my $fh = gensym;
    open $fh, $file
      or croak "can't open log file '$file': $!";

    $self->{'_fh'} = $fh;

    # Open the errors, ignored and continuation streams
    my $src_name = $self->{'_src'}->name;
    if ( $self->{'_converter'}->handle_log_lines ) {
        $self->{'_cont_file'} =
          $self->{'_store'}->storage_file( "continuation", $src_name );
        $self->{'_continuation_fh'} = gensym;

        # We don't append to the continuation log, the new overwrites
        # the old one.
        open $self->{'_continuation_fh'}, "> $self->{'_cont_file'}.new"
          or croak "can't open log continuation storage: $!";

        my $ign_file = $self->{'_store'}->storage_file( "ignored", $src_name );
        $self->{'_ignored_fh'} = gensym;
        open $self->{'_ignored_fh'}, ">> $ign_file"
          or croak "can't open storage for ignored log lines: $!";
    }
    my $err_file = $self->{'_store'}->storage_file( "errors", $src_name );
    $self->{'_error_fh'} = gensym;
    open $self->{'_error_fh'}, ">> $err_file"
      or croak "can't open storage for errors: $!";
}

#------------------------------------------------------------------------
# Method _handle_continuation()
#
# Method which feeds back the saved lines.
sub _handle_continuation {
    my $self = $_[0];

    # Process the continuation lines
    if ( -f $self->{'_cont_file'} ) {
        my $fh = gensym;
        open $fh, "$self->{'_cont_file'}"
          or croak "can't open log continuation storage: $!";

        my $converter = $self->{'_converter'};
        my $line;
        while (defined( $line = <$fh> ) ) {
            chomp $line;

            $self->{'_line_count'}++;
            $converter->process_log_line( $self, $line )
        }

        close $fh;
    }

    # Replace the old continuation by the new one
    rename "$self->{'_cont_file'}.new", $self->{'_cont_file'}
      or croak "failed to rename continuation logs";
}

#------------------------------------------------------------------------
# Method _process_log_lines( $fh )
#
# Method which handles the read a line, process a line loop
sub _process_log_lines {
    my ($self, $fh) = @_;

    my $line;
    my $converter = $self->{'_converter'};
    while (defined ($line = <$fh>)) {
        # Remove DOS and/or UNIX line ending
        $line =~ s/\r?\n?$//;
        $self->{'_line_count'}++;

        $converter->process_log_line( $self, $line )
    }
}

=pod

=head2 dlf_store()

Returns the Lire::DlfStore in which this conversion process is storing
the DLF records.

=cut

sub dlf_store { $_[0]{'_store'} }

=pod

=head2 log_source()

Returns the Lire::LogSource upon which this conversion process is operating.

=cut

sub log_source { $_[0]{'_src'}}

=pod

=head2 line_count()

Returns the number of lines processed. This will 0 in case the DLF
converter process file and not log lines. During a processing, this is
always equals to the line that is currently being converted.

=cut

sub line_count { $_[0]{'_line_count'}}

=pod

=head2 dlf_count()

Returns the number of DLF records created.

=cut

sub dlf_count { $_[0]{'_dlf_count'} }

=pod

=head2 error_count()

Returns the number of errors encountered in the conversion process.

=cut

sub error_count { $_[0]{'_error_count'} }

=pod

=head2 ignored_count()

Returns the number of records which were ignored in the conversion
process.

=cut

sub ignored_count { $_[0]{'_ignored_count'} }

=pod

=head2 saved_count()

Returns the number of lines which were saved for later processing.

=cut

sub saved_count { $_[0]{'_saved_count'} }

=pod

=head1 API FOR THE DLF CONVERTERS

This is the object that encapsulates the Dlf implementation and hides
the complexitity of the storage framework from the DLF converter. It
offers the following methods to the DLf converter.

=head2 write_dlf( $schema, $dlf )

This writes the $dlf DLF record conforming the $schema's schema in the
Lire::DlfStore. The schema is the schema's name (e.g. 'www'). $dlf is
an hash reference. Keys are the schema's field name. Undefined value
means that this field isn't available in that record.

=cut

sub write_dlf {
    my ( $self, $schema, $dlf ) = @_;

    croak "missing schema parameter"
      unless defined $schema;

    croak "missing dlf parameter"
      unless defined $dlf;

    my $s = $self->{'_streams'}{$schema};
    croak "schema $schema wasn't defined by ", $self->{'_converter'}->name,
      " converter"
        unless defined $s;

    $s->write_dlf( $dlf );

    $self->{'_dlf_count'}++;
}

=pod

=head2 save_log_line( $line )

Method that should be used to save $line for a future processing run
of the converter on the same Lire::LogSource.

=cut

sub save_log_line {
    my ( $self, $line ) = @_;

    croak "missing line parameter"
      unless defined $line;

    croak "only DLF converter handling log lines can save log line"
      unless defined $self->{'_continuation_fh'};

    $self->{'_saved_count'}++;

    print {$self->{'_continuation_fh'}} $line, "\n";
}

=pod

=head2 ignored_log_line( $line, [ $reason ] )

Method that can be used by the Lire::DlfConverter to report that the
'$line' log line was ignored during that processing. The reason why
the line was ignored can be given in $reason. For example,
syslog-based converter should use that method to report lines that are
for another 'service' than theirs.

=cut

sub ignore_log_line {
    my ( $self, $line, $reason ) = @_;

    $reason ||= "Unknown reason";

    croak "missing line parameter"
      unless defined $line;

    croak "only DLF converter handling log lines can save log line"
      unless defined $self->{'_ignored_fh'};

    $self->{'_ignored_count'}++;

    print {$self->{'_ignored_fh'}} _ts(), " ", $reason, ":", $line, "\n";
}

#------------------------------------------------------------------------
# Function _ts
#
# Returns a timestamp
sub _ts {
    strftime( '%y-%m-%d %H:%M:%S', localtime )
}

=pod

=head2 error( $error_msg, [ $line ] );

Method that should be used by the Lire::DlfConveter to report that an
error was encountered when processing the Lire::LogSource. $error_msg
should be used to report the nature of the error. The $line parameter
should be used by converter operating on lines to associate the error
message to a particular line.

=cut

sub error {
    my ( $self, $error_msg, $line ) = @_;

    croak "missing error_msg parameter"
      unless defined $error_msg;

    $self->{'_error_count'}++;
    print {$self->{'_error_fh'}} _ts, " ", $error_msg,
      ( defined $line ? (":", $line) : "" ), "\n";
}

# keep perl happy
1;

__END__

=pod

=head1 SEE ALSO

Lire::DlfStore(3pm) Lire::DlfConverter(3pm)

=head1 VERSION

$Id: DlfConverterProcess.pm,v 1.3 2003/10/29 06:19:17 wsourdeau Exp $

=head1 AUTHOR

Francis J. Lacoste <flacoste@logreport.org>

=head1 COPYRIGHT

Copyright (C) 2002 Stichting LogReport Foundation LogReport@LogReport.org

This file is part of Lire.

Lire is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see COPYING); if not, check with
http://www.gnu.org/copyleft/gpl.html or write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

=cut

