package Lire::DlfStore;

use strict;

use vars qw/ $VERSION /;

use Lire::DlfStream;
use Lire::DlfQuery;
use Lire::SQLExt::Registry;

use Errno;
use Fcntl;
use Symbol;

use DBI;

use Carp;

BEGIN {
    ($VERSION)	= '$Revision: 1.18 $' =~ m!Revision: ([.\d]+)!;
};

=pod

=head1 NAME

Lire::DlfStore - Interface to a DLF store

=head1 SYNOPSIS

  use Lire::DlfStore;

  my $store = Lire::DlfStore->open( "mystore", 1 );
  my $avail_streams = $store->dlf_streams;

=head1 DESCRIPTION

=head2 open( $store_path, [ $create ] );

Opens a Lire::DlfStore residing in the directory pointed by
$store_path and returns a Lire::DlfStore instance that can be used to
access its content.

If the store $store_path wasn't created, the method will die unless
the $create parameter was set to true.

If the store is locked by another process, the method will also die.

Trying to open the same store twice from the same process will fail.

=cut

sub open {
    my ( $pkg, $store_path, $create ) = @_;

    my $self = bless { '_store_path' => $store_path,
                       '_dbh' => undef,
                     }, $pkg;

    if ( ! -d $store_path ) {
        croak "DlfStore '$store_path' doesn't exist"
          unless $create;

        $self->_create_store;
    }

    $self->{'_dbh'} = DBI->connect( "dbi:SQLite:dbname=" . 
                                    $self->{'_store_path'} . "/dlf.db", "", "",
                                    { 'RaiseError' => 1,
                                      'AutoCommit' => 0, }
                                  );

    Lire::SQLExt::Registry->register_functions( $self->{'_dbh'} );
    Lire::SQLExt::Registry->register_aggregates( $self->{'_dbh'} );

    my $locked_by = $self->_lock;
    croak "DlfStore '$store_path' is locked by process $locked_by"
      unless $locked_by == $$;

    return $self;
}

=pod

=head2 close()

Calls this method to release the lock held by the current process on
the store. If you don't call it, it will be called automatically when
the store reference goes out of scope, but you better not rely on this
to close your stores.

=cut

sub close {
    my $self = $_[0];

    $self->_unlock;

    # DESTROY calls close() another time
    if ( $self->{'_dbh'} ) {
        $self->{'_dbh'}->commit;
        $self->{'_dbh'}->disconnect;
        delete $self->{'_dbh'};
    }
}

=pod

=head2 path

Returns the path to the store. This is the argument used to create the
store.

=cut

sub path {
    $_[0]{'_store_path'};
}

#------------------------------------------------------------------------
# Method _create_store( )
#
# Creates the DlfStore structure.
sub _create_store {
    my $self = $_[0];


    mkdir $self->{'_store_path'}, 0770
      or croak "can't create DLF store '$self->{'_store_path'}: $!";

    foreach my $d ( qw/continuation ignored errors/ ) {
        $self->create_category( $d );
    }
}

#------------------------------------------------------------------------
# Method _lock()
#
# Acquire a lock on the store. This is done automatically from open()
# It tries three time to acquired the lock before returning. It
# returns the process id of the owner of the lock. To check if the
# lock was acquired the called must check that this is equals to $$.
sub _lock {
    my $self = $_[0];

    # Do not try to acquired another lock.
    return if $self->{'_lock_file'};

    my $lock_file = $self->{'_store_path'} . "/lock";
    my $lock_fh = gensym;
    my $max_try = 3;
    for ( my $try =1; $try <= $max_try; $try++ ) {
        if ( sysopen( $lock_fh, $lock_file, O_EXCL|O_CREAT|O_RDWR) ) {
            print $lock_fh $$, "\n";
            CORE::close $lock_fh;
            $self->{'_lock_file'} = $lock_file;
            return $$;
        }
        if ( CORE::open $lock_fh, $lock_file ) {
            my $lock_pid = <$lock_fh>;
            chomp $lock_pid;

            if (kill( 0, $lock_pid) || $!{'EPERM'} ) {
                return $lock_pid if $try == $max_try;
                sleep 2;
            } else {
                # Stale lock
                unlink( $lock_file ) == 1
                  or croak "can't remove stale lock file '$lock_file': $!";
            }
        }
    }

    die "ASSERTION FAILED: unreachable code";
}

#------------------------------------------------------------------------
# Method _unlock()
#
# Release the lock previously acquired by _lock() method.
sub _unlock {
    my $self = $_[0];

    return unless $self->{'_lock_file'};

    unlink $self->{'_lock_file'};
}

sub DESTROY {
    my $self = $_[0];

    $self->close;
}

=pod

=head1 DLF STREAMS RELATED METHOD

A DlfStore can contains multiple Dlf Stream. Dlf Stream is data
conforming to a DLF schema. The streams are represented by
Lire::DlfStream objects.

=head2 dlf_streams()

List the DLF stream available in the DLF store. This returns a list of
DlfSchema's name.

=cut

sub dlf_streams {
    my $self = $_[0];

    my @streams = ();
    my $sth = $self->{'_dbh'}->table_info( "", "", "dlf_%", "TABLE" );
    $sth->execute;
    while ( my $table_info = $sth->fetchrow_hashref ) {
        next unless $table_info->{'TABLE_NAME'} =~ /^dlf_(.*)/;
        push @streams, $1;
    }
    $sth->finish;

    return @streams;
}

=pod

=head2 has_dlf_stream( $name )

Returns true if the DlfStore contains a DlfStream conforming to the
schema $name.

=cut

sub has_dlf_stream {
    my ( $self, $name ) = @_;

    return grep { $_ eq $name } $self->dlf_streams;
}

=pod

=head2 open_dlf_stream( $name, [$mode], [$sort_spec] )

Returns a Lire::DlfStream object that can be used to a DLF stream in
the schema $name. The $mode parameter can either "r" or "w" and
defaults to "r".

If the mode is "w", the Lire::DlfStream is opened for writing (new Dlf
records can be inserted). If the DlfStore didn't contains a stream
$name, an empty one will be created.

If the mode is "r", the Lire::DlfStream is opened for reading (no DLF
records may be inserted). An exception will be thrown if there is no
stream $name available.

When the stream is opened in "r" mode, a sort order can be specified
by passing a $sort_spec. A $sort_spec is a white-space delimited list
of field names which should specify the sort order. Reverse sort order
can be specified by prefixing the field's name by '-'.

=cut
sub open_dlf_stream {
    my ( $self, $name, $mode, $sort_spec ) = @_;

    $mode ||= "r";

    croak "mode should be either 'r' or 'w' not '$mode'"
      unless $mode eq 'r' or $mode eq 'w';

    croak "no DLF stream '$name' in this store"
      if ( $mode eq 'r' && ! $self->has_dlf_stream( $name ) );

    croak "only 'r' mode can use a sort_spec"
      if ( $mode ne 'r' && $sort_spec );

    return new Lire::DlfStream( $self, $name, $mode, $sort_spec );

}

=pod

=head1 PERSISTENT STORAGE METHODS

A DlfStore can be used to store persistent for converters and
analyzers. The interface is very simple. Client can ask the store for
path to place where they can hold their persistent data in files. Each
file is defined in one category. Pre-defined category are 'dlf',
'errors', 'ignored' and 'continuation' used to cover the need of the
DLF converters and analyzers API. Other categories can be created by
using the create_category() method.

=head2 create_category( $category )

Creates the space for a category named $category in the DlfStore. The
category name should contains only alphanumerical character (plus the
'underscore'). This method will throw an exception on error.

=cut

sub create_category {
    my ( $self, $category ) = @_;

    croak "missing category parameter" 
      unless defined $category;

    croak "invalid category name: $category"
      unless $category =~ /^\w+$/;

    my $dir = "$self->{'_store_path'}/$category";
    return if -d $dir;

    mkdir $dir, 0777
      or croak "failed creating cateogry '$category' of DLF store '$self->{'_store_path'}': $!";
}

=pod

=head2 categories()

Returns the list of categories defined in this store.

=cut

sub categories {
    my $self = $_[0];

    opendir STORE_DIR, $self->{'_store_path'}
      or croak "opendir $self->{'_store_path'} failed: $!\n";
    my @cat = grep { $_ !~ /^\.\.?$/ && -d $self->{'_store_path'} . "/$_" }
      readdir STORE_DIR;
    closedir STORE_DIR;

    return @cat;
}

=pod

=head2 has_category( $category )

Returns true if the category $category exists in the current store.

=cut

sub has_category {
    my ( $self, $category ) = @_;

    croak "missing category parameter"
      unless defined $category;

    return -d "$self->{'_store_path'}/$category";
}

=pod

=head2 storage_files( $category )

Returns the name of the files available in the $category category.

=cut

sub storage_files {
    my ( $self, $category ) = @_;

    croak "missing 'category' parameter"
      unless defined $category;

    croak "category '$category' doesn't exist"
      unless $self->has_category( $category );

    my $cat_path = "$self->{'_store_path'}/$category";
    opendir CAT_DIR, $cat_path
      or croak "opendir '$cat_path' failed: $!";
    my @files = grep { -f $cat_path . "/" . $_ } readdir CAT_DIR;
    closedir CAT_DIR;

    return @files;
}

=pod

=head2 storage_file( $category, $name )

Returns the name of file that can be used to retrieve or store data in
the $category category under $name. Mode is used to indicate if the

This method will throw an exception if the $category doesn't exist.

=cut

sub storage_file {
    my ( $self, $category, $name ) = @_;

    croak "missing 'category' parameter"
      unless defined $category;

    croak "missing 'name' parameter"
      unless defined $name;

    croak "category '$category' doesn't exist"
      unless $self->has_category( $category );

    croak "invalid 'name' parameter: $name. Names can only contain letters, digits, underscores, hyphens and periods."
      unless $name =~ /^[\w.-]+$/;


    return "$self->{'_store_path'}/$category/$name";
}

sub _dbh {
    return $_[0]{'_dbh'};
}

# keep perl happy
1;

__END__

=pod

=head1 SEE ALSO

Lire::DlfConverter(3pm)

=head1 VERSION

$Id: DlfStore.pm,v 1.18 2003/10/28 16:26:10 flacoste Exp $

=head1 AUTHOR

Francis J. Lacoste <flacoste@logreport.org>

=head1 COPYRIGHT

Copyright (C) 2002 Stichting LogReport Foundation LogReport@LogReport.org

This file is part of Lire.

Lire is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see COPYING); if not, check with
http://www.gnu.org/copyleft/gpl.html or write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

=cut

