package Lire::LogSource;

use strict;

use vars qw/ $VERSION /;

use Lire::DlfConverterManager;

use POSIX qw/strftime/;

use Carp;

BEGIN {
    ($VERSION)	= '$Revision: 1.6 $' =~ m!Revision: ([.\d]+)!;
};

=pod

=head1 NAME

Lire::LogSource - Object used to represent a source of log data

=head1 SYNOPSIS

  use Lire::LogSource;

  my $src = new Lire::LogSource( "name",
                                 'pattern'    => "/var/log/messsages",
                                 'period'     => "unique",
                                 'processor'  => $name,
                                );

  my $file = $src->file,

=head1 DESCRIPTION

The Lire::LogSource object is used to represent a source of log data.

=head1 new( $name, ... );

Create a new Lire::LogSource object. As mandatory parameter, it takes
the name of the log source. This is an identifier consisting of
alphanumerical characters. It may also contain underscores ('_').

Addition parameters:

=over

=item pattern

The path to the file that contains the log data. This filename can
contain strftime(3) patterns that will be substituted to obtain the
name of the file. (Useful for automatic rotation).

=item period

The periodicity to which this LogSource should be processed. Defaults
to 'unique' which means this is a one-shot LogSource. Other
recognized values are 'daily', 'weekly' and 'monthly'.

=item converter

The name of the converter that can be used to process the LogSource.
This must be known to the Lire::DlfConverterManager.

=back

=cut

sub new {
    my ($pkg, $name, %args) = @_;

    croak "missing name parameter" unless defined $name;
    croak "invalid name parameter: $name" unless $name =~ /^\w+$/;

    $args{'period'} ||= "unique";

    my $self = bless { '_name' => $name,
                       '_converter' => undef,
                       '_pattern' => undef,
                       '_period' => undef,
                     }, ref $pkg || $pkg;

    foreach my $p ( qw/pattern period converter/ ) {
        no strict 'refs';
        $self->$p( $args{$p} ) if exists $args{$p};
    }

    return $self;
}

=pod

=head2 name()

Returns the name of this LogSource.

=cut

sub name { $_[0]{'_name'} };

=pod

=head2 period( [$new_period] )

With no argument, returns the period of this LogSource. With a
parameter, the LogSource's period is set to this new value.

=cut

sub period {
    my ( $self, $period ) = @_;

    if ( @_ == 2 ) {
        croak "missing period parameter"
          unless defined $period;

        croak "invalid period parameter: $period ('unique', 'daily', 'weekly', 'monthly' )"
          unless $period =~ m/^(unique|daily|weekly|monthly)$/;

        $self->{'_period'} = $period;
    }

    $self->{'_period'};
}

=pod

=head2 file( [$time] )

Returns the file to be used. The optional $time parameter (which
defaults to now) is used to compute the file name. To find the file to
use for yesterday's data, one could pass time - 86400.

=cut

sub file {
    my ( $self, $time ) = @_;

    $time ||= time;

    return strftime $self->{'_pattern'}, localtime $time;
}

=pod

=head2 pattern( [$new_file] )

Returns the file pattern to use as log data source. This pattern may
contain strftime(3) substitutions. If an argument is used, it will
change the file to use.

=cut

sub pattern {
    my ( $self, $pattern ) = @_;

    if ( @_ == 2 ) {
        croak "missing pattern parameter" 
          unless defined $pattern;

        $self->{'_pattern'} = $pattern;
    }

    $self->{'_pattern'};
}

=pod

=head2 converter( [$new_name] )

Returns the name of the converter that should be used for this
LogSource.

=cut

sub converter {
    my ( $self, $converter ) = @_;

    if ( @_ == 2 ) {
        croak "missing converter parameter"
          unless defined $converter;

        $self->{'_converter'} = $converter;
    }

    $self->{'_converter'};
}

# keep perl happy
1;

__END__

=pod

=head1 SEE ALSO

Lire::DlfStore(3pm) Lire::DlfConverter(3pm)

=head1 VERSION

$Id: LogSource.pm,v 1.6 2003/10/29 06:19:17 wsourdeau Exp $

=head1 AUTHOR

Francis J. Lacoste <flacoste@logreport.org>

=head1 COPYRIGHT

Copyright (C) 2002, 2003 Stichting LogReport Foundation LogReport@LogReport.org

This file is part of Lire.

Lire is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see COPYING); if not, check with
http://www.gnu.org/copyleft/gpl.html or write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

=cut

