package Lire::Test::TestCase;

use strict;

use base qw/Test::Unit::TestCase/;

use Lire::Config;
use Lire::Test::HashConfig;

use Test::Unit::Failure;
use Test::Unit::Error;

=pod

=head1 NAME

Lire::Test::TestCase - Common base for Lire test cases.

=head1 SYNOPSIS

 use base qw/Lire::Test::TestCase/;

 sub set_up {
     my $self = shift->SUPER::set_up();

     $self->{'cfg'}{'lr_schemas_path'} = [ $self->{'tmpdir'} ];
 }

=head1 DESCRIPTION

This package can be used to write test cases for Lire instead of
inheriting directly from Test::Unit::TestCase.

The default set_up method will create a Lire::Test::HashConfig object
and set it up in the configuration framework. This makes it easy to
set configuration variables to arbitrary values in the set_up()
method.

If you override set_up() and tear_down() be sure to call the SUPER::
implementation.

This subclass also defined a new kind of assertion which make sure
that a snippet of code died with a proper error message.

=head2 assert_died( $test, $regex, [ $msg ] )

Execute $test and fail unless it dies with a message matching $regex.
$test should be a reference to CODE. $regex should be a Regexp ref.

=head1 SEE ALSO

Test::Unit::TestCase(3pm)

=head1 VERSION

$Id: TestCase.pm,v 1.5 2003/10/29 06:19:19 wsourdeau Exp $

=head1 AUTHORS

Francis J. Lacoste <flacoste@logreport.org>

=head1 COPYRIGHT

Copyright (C) 2003 Stichting LogReport Foundation LogReport@LogReport.org

This file is part of Lire.

Lire is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see COPYING); if not, check with
http://www.gnu.org/copyleft/gpl.html or write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

=cut

sub set_up {
    my $self = $_[0];

    $self->{'_saved_cfg'} = $Lire::Config::SINGLETON;
    $self->{'cfg'} = new Lire::Test::HashConfig( {} );
    $Lire::Config::SINGLETON = $self->{'cfg'};

    return $self;
}

sub tear_down {
    my $self = $_[0];

    $Lire::Config::SINGLETON = $self->{'_saved_cfg'};

    return $self;
}

sub assert_died {
    my ( $self, $test, $regex, $msg ) = @_;

    $self->_error( "missing 'test' parameter"  )
      unless defined $test;

    $self->_error( "missing 'regex' parameter"  )
      unless defined $regex;

    $self->_error( "'test' parameter should be a CODE reference" )
      unless ref $test eq 'CODE';

    $self->_error( "'regex' parameter should be a Regexp reference" )
      unless ref $regex eq 'Regexp';

    eval {  $test->() };
    $self->_fail( defined $msg ? $msg : "expected test to die" )
      unless $@;

    $self->_fail( defined $msg ? $msg : "expected die message to match $regex, got '$@'" )
      unless $@ =~ /$regex/;
}

sub _fail {
    my $self = shift;
    my($asserter,$file,$line) = caller(1);
    my $message = join '', @_;
    Test::Unit::Failure->throw(-text => $message,
			       '-object' => $self,
			       '-file' => $file,
			       '-line' => $line);
}

sub _error {
    my $self = shift;
    my($asserter,$file,$line) = caller(1);
    my $message = join '', @_;
    Test::Unit::Error->throw(-text => $message,
                             '-object' => $self,
                             '-file' => $file,
                             '-line' => $line);
}

1;
